/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

/*
  Copyright (C) 2009 Bernd Buschinski <b.buschinski@web.de>
*/

#include "whiteboardglobals.h"

namespace Konversation
{
    namespace DCC
    {
        const int WhiteBoardGlobals::MaxImageSize = 2048;
        const int WhiteBoardGlobals::MaxPenWidth = 20;
    }
}
